import 'dart:async';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:camera/camera.dart';
import 'package:flutter/services.dart';
import 'package:rwa_camera/CameraModule/editPreview.dart';
import 'dart:io';
import 'package:wechat_assets_picker/wechat_assets_picker.dart';
import 'listCameraFlashMode.dart';
export 'package:camera/camera.dart';
import 'package:toast/toast.dart';

CameraController? _cameraController;

enum UiStyle{
  og,smooth,overlay,cropOverlay
}
enum PreviewType{
  og,smooth,crop
}

double cameraAspectRatio = 1;
class OpenCamera extends StatefulWidget {
final int totalQuestion;
final bool switchCamera;
final UiStyle uiStyle;
final PreviewType previewType;
final ResolutionPreset resolutionPreset;
  OpenCamera(
     { this.resolutionPreset = ResolutionPreset.max,
    Key? key,this.totalQuestion = 1,this.switchCamera = false,this.uiStyle = UiStyle.og,this.previewType = PreviewType.smooth
  }) : super(key: key);

  @override
  _OpenCameraState createState() => _OpenCameraState();
}

class _OpenCameraState extends State<OpenCamera> {
  List<XFile> listImageFile = [];
  late File file;
  late int cameraType;
  late int flashMode;
  List<CameraDescription>? cameras;
  ListCameraFlashMode _listCameraFlashMode = ListCameraFlashMode();
  late double fullWidth;
  late double fullHeight;

  @override
  void initState() {
    super.initState();
    cameraType = 0;
    flashMode = 0;
    getCamera();

  }
  void getCamera()async{
    cameras = await availableCameras();
    print(cameras![0].sensorOrientation);
    setState(() {

    });
  }
  @override
  Widget build(BuildContext context) {
    SystemChrome.setPreferredOrientations([
      DeviceOrientation.portraitUp,
      DeviceOrientation.portraitDown,
    ]);
    fullWidth = MediaQuery.of(context).size.width;
    fullHeight = MediaQuery.of(context).size.height;
    if(cameras == null){
      return Container();
    }
    return WillPopScope(
      onWillPop: () async {
        Navigator.pop(context, listImageFile);
        return true;
      },
      child: Scaffold(
        body: Stack(
          children: [
            CameraApp(cameras![0], UniqueKey,widget.resolutionPreset),
            (widget.uiStyle == UiStyle.overlay)?Column(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: [
                Container(
                  width: fullWidth,
                  height: (fullHeight-fullWidth*4/3)/2,
                  color: Colors.black,
                ),

                Container(
                  width: fullWidth,
                  height: (fullHeight-fullWidth*4/3)/2,
                  color: Colors.black,
                )
              ],
            ):Container(),
            Positioned(
              bottom: 0,
              child: Container(
                height: 100,
                width: MediaQuery.of(context).size.width,
                //color: Color.fromRGBO(0, 0, 0, .5),
                child: Builder(builder: (context){
                  return Row(
                    mainAxisAlignment: MainAxisAlignment.spaceAround,
                    children: [
                      Expanded(flex: 1,child: doneClick()),
                      Expanded(flex: 1,child: getGallery()),
                      Expanded(flex: 1,child: cameraControl()),
                      Expanded(flex: 1,child: switchCamera()),
                      Expanded(flex: 1,child: flashButtonSwitch()),
                    ],
                  );
                } ,
                ),
              ),
            ),
          ],
        ),
      ),
    );
  }

  GestureDetector getGallery(){
    return GestureDetector(onTap: () async {
      if(widget.totalQuestion-listImageFile.length ==0){
        Navigator.pop(context, listImageFile);
        return;
      }
      final List<AssetEntity>? receivedImage = await AssetPicker.pickAssets(context,
        maxAssets: widget.totalQuestion-listImageFile.length,
        textDelegate: EnglishTextDelegate(),
      );
      if(receivedImage == null?false:receivedImage.length >0){
        for(var i =0;i<receivedImage.length;i++){
          this.listImageFile.add(XFile((await receivedImage[i].file)!.path));
        }
        Navigator.pop(context, listImageFile);
      }
    },child: getGalleryIcon(widget.uiStyle));
  }
  GestureDetector flashButtonSwitch() {
    return GestureDetector(
                          onTap: () {

                            if (flashMode == _listCameraFlashMode.listCameraFlashMode.length - 1) {
                              flashMode = 0;
                            } else {
                              flashMode = flashMode + 1;
                            }
                            _cameraController!.setFlashMode(_listCameraFlashMode.listCameraFlashMode[flashMode].flashMode);

                            setState(() {});
                          },
                          child: getFlash(widget.uiStyle));
  }
  GestureDetector switchCamera() {
    return GestureDetector(
        onTap: () {
          cameraType = cameraType == 0 ? 1 : 0;
          _cameraController = CameraController(
              cameras![cameraType], widget.resolutionPreset);
          _cameraController!.initialize().then((_) {
            setState(() {});
          });
        },
        child: getSwitchCameraIcon(widget.uiStyle));
  }
  GestureDetector cameraControl() {

    return GestureDetector(
        onTap: () async{
          if(widget.totalQuestion !=1){
            Toast.show("Captured", context, duration: Toast.LENGTH_SHORT, gravity:  Toast.CENTER);
          }
          Future<XFile?> takePicture() async {
            final CameraController? cameraController = _cameraController;
            if (cameraController == null || !cameraController.value.isInitialized) {
              return null;
            }

            if (cameraController.value.isTakingPicture) {
              // A capture is already pending, do nothing.
              return null;
            }

            try {
              XFile file = await cameraController.takePicture();

              return file;
            } on CameraException catch (e) {
              return null;
            }
          }

          await takePicture().then((XFile? file) {
            if (file != null) {
              if(widget.totalQuestion ==listImageFile.length){
                listImageFile.removeAt(0);
              }
              listImageFile.add(file);
            }

            setState(() {});
          });

          if(widget.totalQuestion ==1){
            dynamic listResult = await  Navigator.push(
                context,
                new MaterialPageRoute(
                    builder: (context) => new EditPreview(
                      listImageFile[0],
                    )));

            if(listResult[0]){
              listImageFile.clear();
              listImageFile.add(listResult[1]);
              Navigator.pop(context, listImageFile);

            }
          }
        },
        child: getCameraIcon(widget.uiStyle));
  }
  GestureDetector doneClick(){
    return GestureDetector(
      onTap: (){
        Navigator.pop(context, listImageFile);
      },
      child: getDoneButton(widget.uiStyle),
    );
  }

  Widget getDoneButton(UiStyle uiStyle){
    switch (uiStyle){
      case UiStyle.og:
        return Container(
          alignment: Alignment.center,
          child: ClipRRect(
            borderRadius: BorderRadius.all(Radius.circular(20)),
            child: Container(
              height: 40,
              width: 40,
              color: Colors.white,
              alignment: Alignment.center,
              child: Stack(
                children: [
                  Positioned(
                      top: 5,
                      right: 5,
                      child: Text(listImageFile.length.toString(),style: TextStyle(fontSize: 12),)),
                  Center(
                    child: Icon(Icons.done),
                  ),
                ],
              ),),
          ),
        );
        break;
      case UiStyle.smooth:
        return Container(decoration: BoxDecoration(shape: BoxShape.circle,color: Colors.black26,),width:50,height: 50,alignment:Alignment.center,child: Icon(Icons.done,color: Colors.white,));
        break;
      case UiStyle.overlay:
        return Container(decoration: BoxDecoration(shape: BoxShape.circle,color: Colors.black26,),width:50,height: 50,alignment:Alignment.center,child: Icon(Icons.done,color: Colors.white,));
        break;
      case UiStyle.cropOverlay:
        return Container(decoration: BoxDecoration(shape: BoxShape.circle,color: Colors.black26,),width:50,height: 50,alignment:Alignment.center,child: Icon(Icons.done,color: Colors.white,));
        break;
    }
  }
  Widget getGalleryIcon(UiStyle uiStyle){
    switch (uiStyle){
      case UiStyle.og:
        return Container(
          alignment: Alignment.center,
          child: Container(
            height:40,
            width:40,
            child: ClipRRect(

                borderRadius:BorderRadius.all(Radius.circular(20)),

                child: listImageFile.length>0?Image.file(File(listImageFile[listImageFile.length-1].path),fit:BoxFit.fill):Container(color: Colors.blue,)),
          ),
        );
        break;
      case UiStyle.smooth:
        return Container(decoration: BoxDecoration(shape: BoxShape.circle,color: Colors.black26,),width:50,height: 50,alignment:Alignment.center,child: Image.asset('assets/attach@3x.png',package: 'rwa_camera',width: 20,height: 20,));
        break;
      case UiStyle.overlay:
        return Container(decoration: BoxDecoration(shape: BoxShape.circle,color: Colors.black26,),width:50,height: 50,alignment:Alignment.center,child: Image.asset('assets/attach@3x.png',package: 'rwa_camera',width: 20,height: 20,));
        break;
      case UiStyle.cropOverlay:
        return Container(decoration: BoxDecoration(shape: BoxShape.circle,color: Colors.black26,),width:50,height: 50,alignment:Alignment.center,child: Image.asset('assets/attach@3x.png',package: 'rwa_camera',width: 20,height: 20,));
        break;
    }
  }
  Widget getFlash(UiStyle uiStyle){
    switch (uiStyle){
      case UiStyle.og:
        return Icon(_listCameraFlashMode.listCameraFlashMode[flashMode].iconData, color: Colors.white);
        break;
      case UiStyle.smooth:
        return Container(decoration: BoxDecoration(shape: BoxShape.circle,color: Colors.black26,),width:50,height: 50,alignment:Alignment.center,child: Icon(_listCameraFlashMode.listCameraFlashMode[flashMode].iconData, color: Colors.white,size: 25,));
        break;
      case UiStyle.overlay:
        return Container(decoration: BoxDecoration(shape: BoxShape.circle,color: Colors.black26,),width:50,height: 50,alignment:Alignment.center,child: Icon(_listCameraFlashMode.listCameraFlashMode[flashMode].iconData, color: Colors.white,size: 25,));
        break;
      case UiStyle.cropOverlay:
        return Container(decoration: BoxDecoration(shape: BoxShape.circle,color: Colors.black26,),width:50,height: 50,alignment:Alignment.center,child: Icon(_listCameraFlashMode.listCameraFlashMode[flashMode].iconData, color: Colors.white,size: 25,));
        break;
    }
  }
  Widget getSwitchCameraIcon(UiStyle uiStyle){
    switch (uiStyle){
      case UiStyle.og:
        return Icon(Icons.switch_camera,color: Colors.white,);
        break;
    //Group 794@2x.png
      case UiStyle.smooth:
        return Container(decoration: BoxDecoration(shape: BoxShape.circle,color: Colors.black26,),width:50,height: 50,alignment:Alignment.center,child: Image.asset('assets/Group 794@2x.png',package: 'rwa_camera',width: 20,height: 20,));
        break;
      case UiStyle.overlay:
        return Container(decoration: BoxDecoration(shape: BoxShape.circle,color: Colors.black26,),width:50,height: 50,alignment:Alignment.center,child: Image.asset('assets/Group 794@2x.png',package: 'rwa_camera',width: 20,height: 20,));
        break;
      case UiStyle.cropOverlay:
        return Container(decoration: BoxDecoration(shape: BoxShape.circle,color: Colors.black26,),width:50,height: 50,alignment:Alignment.center,child: Image.asset('assets/Group 794@2x.png',package: 'rwa_camera',width: 20,height: 20,));
        break;
    }
  }
  Widget getCameraIcon(UiStyle uiStyle){
    switch (uiStyle){
      case UiStyle.og:
        return Icon(
          Icons.camera,
          color: Colors.white,
          size: 60,
        );
        break;
      case UiStyle.smooth:
        return Image.asset('assets/camera.png',package: 'rwa_camera',);
        break;
      case UiStyle.overlay:
        return Image.asset('assets/camera.png',package: 'rwa_camera',);
        break;
      case UiStyle.cropOverlay:
        return Image.asset('assets/camera.png',package: 'rwa_camera',);
        break;
    }
  }
}

class CameraApp extends StatefulWidget {
  CameraDescription cameras;
  ResolutionPreset resolutionPreset;
  CameraApp(this.cameras, key,this.resolutionPreset);

  @override
  _CameraAppState createState() => _CameraAppState();
}

class _CameraAppState extends State<CameraApp> {
  @override
  void initState() {
    super.initState();
    _cameraController = CameraController(widget.cameras, widget.resolutionPreset);
    _cameraController!.initialize().then((_) {
      cameraAspectRatio = _cameraController!.value.aspectRatio;
      if (!mounted) {
       setState(() {

       });
        return;
      }
      setState(() {});
    });
  }

  @override
  void dispose() {
    _cameraController?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (!_cameraController!.value.isInitialized) {
      return Container();
    }
    return MaterialApp(
      home: Container(
        color: Colors.black,
        alignment: Alignment.center,
          height: MediaQuery.of(context).size.width*cameraAspectRatio,
          width: MediaQuery.of(context).size.width,
          child: CameraPreview(_cameraController!)),
    );
  }
}

class EnglishTextDelegate implements AssetsPickerTextDelegate {
  factory EnglishTextDelegate() => _instance;

  EnglishTextDelegate._internal();

  static final EnglishTextDelegate _instance = EnglishTextDelegate._internal();

  @override
  String confirm = 'Confirm';

  @override
  String cancel = 'Cancel';

  @override
  String edit = 'Edit';

  @override
  String gifIndicator = 'GIF';

  @override
  String heicNotSupported = 'Unsupported HEIC asset type.';

  @override
  String loadFailed = 'Load failed';

  @override
  String original = 'Origin';

  @override
  String preview = 'Preview';

  @override
  String select = 'Select';

  @override
  String unSupportedAssetType = 'Unsupported HEIC asset type.';

  @override
  String durationIndicatorBuilder(Duration duration) =>
      AssetsPickerTextDelegate.defaultDurationIndicatorBuilder(duration);
}